/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.framework.util;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 *
 * @version 0.0.20, 04/07/1998
 * @author  Federico Barbieri <fede@apache.org>
 * @author  Stefano Mazzocchi <mazzocch@mbox.systemy.it>
 */
public class Circuit
{
    protected Hashtable m_map;

    public Circuit()
    {
        m_map = new Hashtable();
    }

    public void addNode( final String name )
    {
        if( null == m_map.get( name ) )
        {
            m_map.put( name, new Node( name ) );
        }
    }

    public void removeNode( final String name )
    {
        String tmp = null;
        Enumeration e = m_map.keys();

        while( e.hasMoreElements() )
        {
            tmp = (String)e.nextElement();

            if( !tmp.equals( name ) )
            {
                try { unlink( tmp, name ); }
                catch( final CircuitException ce) {}
                try { unlink( name, tmp ); }
                catch( final CircuitException ce ) {}
            }

        }

        m_map.remove( name );
    }

    public void link( final String parent, final String child )
        throws CircuitException
    {
        Node tempNode = null;
        Node pnode = (Node)m_map.get( parent );
        Node cnode = (Node)m_map.get( child );
        if( null == pnode )
        {
            throw new CircuitException( "Unknown node " + parent );
        }
        else if( null == cnode )
        {
            throw new CircuitException( "Unknown node " + child );
        }
        else if( pnode.isChildOf( cnode ) )
        {
            throw new CircuitException( "Loop! Node " + parent +
                                        " is already child of node " + child );
        }
        else
        {
            final Enumeration e = m_map.elements();

            while( e.hasMoreElements() )
            {
                tempNode = (Node)e.nextElement();
                if( tempNode.isChildOf( cnode ) )
                {
                    tempNode.m_parents.addAll( pnode.m_parents );
                }
            }
        }
    }

    public void unlink( final String parent, final String child )
        throws CircuitException
    {
        Node cnode = (Node)m_map.get( child );
        Node pnode = (Node)m_map.get( parent );

        if( cnode.m_parents.contains( pnode ) )
        {
            Node tempNode = null;
            Enumeration e = m_map.elements();

            while( e.hasMoreElements() )
            {
                tempNode = (Node)e.nextElement();

                if( tempNode.m_parents.contains( cnode ) )
                {
                    tempNode.m_parents.removeAll( pnode.m_parents );
                }
            }
        }
        else
        {
            throw new CircuitException( "Node " + parent + " is not parent of node " + child );
        }
    }

    public Vector getAncestors()
    {
        Vector ancestors = new Vector();
        String name = null;
        Node tempNode = null;
        Enumeration e = m_map.keys();

        while( e.hasMoreElements() )
        {
            name = (String)e.nextElement();
            tempNode = (Node)m_map.get( name );

            if( 1 == tempNode.m_parents.size() )
            {
                ancestors.addElement( name );
            }
        }

        return ancestors;
    }

    public String getAncestor()
    {
        String name = null;
        Node tempNode = null;
        Enumeration e = m_map.keys();

        while( e.hasMoreElements() )
        {
            name = (String)e.nextElement();
            tempNode = (Node)m_map.get( name );

            if( 1 == tempNode.m_parents.size() )
            {
                return name;
            }
        }

        return null;
    }

    public boolean isEmpty()
    {
        return m_map.isEmpty();
    }

    protected final class Node
    {
        protected Vector m_parents;
        protected String m_name;

        protected Node( final String name )
        {
            m_parents = new Vector( 5 );
            m_parents.addElement( this );
            m_name = name;
        }

        protected boolean isChildOf( final Node parent )
        {
            return m_parents.contains( parent );
        }

        public String toString()
        {
            StringBuffer buffer = new StringBuffer();
            Enumeration e = m_parents.elements();
            buffer.append( m_name + "[" );

            while( e.hasMoreElements() )
            {
                buffer.append(((Node) e.nextElement()).m_name + " ");
            }

            buffer.append("]");
            return buffer.toString();
        }
    }

    public final class CircuitException
        extends RuntimeException
    {
        public CircuitException()
        {
        }

        public CircuitException( final String message )
        {
            super( message );
        }
    }

    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        String name = null;
        Node tempNode = null;
        Enumeration e = m_map.keys();

        while( e.hasMoreElements() )
        {
            name = (String)e.nextElement();
            tempNode = (Node)m_map.get( name );
            buffer.append( name + "(" + ( tempNode.m_parents.size() - 1 ) + ") " );
        }

        return buffer.toString();
    }
}

